categories = [
    {"from": "AAComm", "name": "ComAPI", "enable": True},
    {"from": "AAComm", "name": "CommAPIWrapper", "enable": False}, #example of no import
]
SharedCategories = [
    {"from": "AAComm.Shared", "name": "ChannelType", "enable": True},
    {"from": "AAComm.Shared", "name": "ConnectResult", "enable": True},
]
Servicescategories = [
    {"from": "AAComm.Services", "name": "ConnectionData", "enable": True},
]
Extensionscategories = [
    {"from":"AAComm.Extensions", "name":"AACommDownloadFW","enable": True},
    {"from":"AAComm.Extensions", "name":"AACommDownloadUP","enable": True},
]
categories.extend(SharedCategories)
categories.extend(Servicescategories)
categories.extend(Extensionscategories)


import os
import shutil
import subprocess

def download_nuget(version: str = "", update: bool = False) -> None:
    nuget_path = os.path.join(os.path.dirname(__file__), 'aacommpyDownloader-main', 'nuget.exe')
    installed = False
    for dirname in os.listdir(os.path.dirname(nuget_path)):
        if dirname.startswith('Agito.AAComm.') and os.path.isdir(os.path.join(os.path.dirname(nuget_path), dirname)):
            installed = True
            old_version = dirname.split('.')[2:]
            old_version = '.'.join(old_version)
            break
    if update and installed:
        shutil.rmtree(os.path.join(os.path.dirname(nuget_path), f'Agito.AAComm.{old_version}'))
    nuget_cmd = [nuget_path, 'install', 'Agito.AAComm', '-OutputDirectory', os.path.join(os.path.dirname(nuget_path)), '-Source', 'https://api.nuget.org/v3/index.json']
    if version != "":
        nuget_cmd.extend(['-Version', version])
    subprocess.run(nuget_cmd, check=True)
    for dirname in os.listdir(os.path.dirname(nuget_path)):
        if dirname.startswith('Agito.AAComm.') and os.path.isdir(os.path.join(os.path.dirname(nuget_path), dirname)):
            new_version = dirname.split('.')[2:]
            new_version = '.'.join(new_version)
            source_dir = os.path.join(os.path.dirname(nuget_path), f'Agito.AAComm.{new_version}/build/AACommServer')
            dest_dir = os.path.dirname(__file__)
            shutil.copy2(os.path.join(source_dir, 'AACommServer.exe'), dest_dir)
            shutil.copy2(os.path.join(source_dir, 'AACommServerAPI.dll'), dest_dir)
            source_dir2 = os.path.join(os.path.dirname(nuget_path), f'Agito.AAComm.{new_version}/lib/net5.0')
            shutil.copy2(os.path.join(source_dir2, 'AAComm.dll'), dest_dir)
            break
    return None
def nuget_version() -> str:
    nuget_path = os.path.join(os.path.dirname(__file__), 'aacommpyDownloader-main', 'nuget.exe')
    installed = False
    latest_version = None
    for dirname in os.listdir(os.path.dirname(nuget_path)):
        if dirname.startswith('Agito.AAComm.') and os.path.isdir(os.path.join(os.path.dirname(nuget_path), dirname)):
            installed = True
            version = dirname.split('.')[2:]
            latest_version = '.'.join(version)
            print(f"The installed version of Agito.AAComm is {latest_version}.")
            break

    if not installed:
        raise RuntimeError('Agito.AAComm package is not installed.')
    
    return latest_version
def update_nuget() -> None:
    download_nuget(update=True)
    return None
def dotnetfw(version: str = "net40") -> None:
    latest_version = nuget_version()
    source_dir = os.path.join(os.path.dirname(__file__), 'aacommpyDownloader-main', f'Agito.AAComm.{latest_version}')
    dest_dir = os.path.dirname(__file__)    
    if version == "net5.0":
        source_dir = os.path.join(source_dir, 'lib', 'net5.0')
    elif version == "net40":
        source_dir = os.path.join(source_dir, 'lib', 'net40')
    elif version == "net46":
        source_dir = os.path.join(source_dir, 'lib', 'net46')
    elif version == "net48":
        source_dir = os.path.join(source_dir, 'lib', 'net48')
    elif version == "netcoreapp3.1":
        source_dir = os.path.join(source_dir, 'lib', 'netcoreapp3.1')
    else:
        raise ValueError("Invalid .NET framework version specified.")    
    dll_path = os.path.join(source_dir, 'AAComm.dll')
    if not os.path.isfile(dll_path):
        raise FileNotFoundError(f"Could not find AAComm.dll in {source_dir}.")
    shutil.copy2(dll_path, dest_dir)
    print(f"The current .NET framework target of AAComm is {version}")
    return None
def get_dotnet_versions():
    try:
        result = subprocess.run(['reg', 'query', 'HKLM\\SOFTWARE\\Microsoft\\NET Framework Setup\\NDP'], capture_output=True, text=True, check=True)
        output_lines = result.stdout.strip().split('\n')
        versions = []
        for line in output_lines:
            if '\\NDP\\v' in line:
                version = line.split('\\')[-1][1:]
                versions.append(version)
        return versions
    except subprocess.CalledProcessError as e:
        print(f"Error occurred while retrieving .NET Framework versions: {e}")
        return []

def check_dotnet_versions():
    target_versions = ["4.0", "4.6", "4.8", "5.0"]
    dotnet_versions = get_dotnet_versions()
    matching_versions = [dotnetfw_from_dotnet_version(version) for version in target_versions if version in dotnet_versions]

    if not matching_versions:
        print("No proper .NET Framework versions found.")
        print("Please install one of the following .NET Framework versions:")
        for version in target_versions:
            print(version)
    else:
        print("Installed Dotnet framework versions which can be used with AAComm:")
        for version in matching_versions:
            print(version)

    return matching_versions

def dotnetfw_from_dotnet_version(dotnet_version):
    version_mapping = {
        "4.0": "net40",
        "4.6": "net46",
        "4.8": "net48",
        "5.0": "net5.0"
    }

    return version_mapping.get(dotnet_version)


import clr
import os

path = os.path.join(os.path.dirname(__file__), 'AAComm.dll')
clr.AddReference(path)

from AAComm import Services
from AAComm import CommAPI
from AAComm import Shared

api = CommAPI()

status = CommAPI.StartAACommServer()

if status != "":
    print(status)
else:
    # Access the static variable IsAACommServerRunning
    is_running = CommAPI.IsAACommServerRunning
    print(f"AACommServer is running: {is_running}")

cData = Services.ConnectionData()
cData.ControllerType = Shared.ProductTypes.AGM800_ID
cData.CommChannelType = Shared.ChannelType.Ethernet
cData.ET_IP_1 = 172
cData.ET_IP_2 = 1
cData.ET_IP_3 = 1
cData.ET_IP_4 = 101
cData.ET_Port = 5000

res = api.Connect(cData)
print(res)

# from System import EventHandler
# from AAComm.Services import AACommEventArgs

# def on_reply_received(sender, event_args):
#     print("Reply received:", event_args.Message)


# OnReplyReceived = EventHandler[AACommEventArgs](on_reply_received)
# msg = Services.AACommMessage(OnReplyReceived, "AMotorOn")




import clr
import os
from category import categories

path = os.path.join(os.path.dirname(__file__), 'AAComm.dll')
clr.AddReference(path)

for category in categories:
    if category["enable"]:
        exec(f"from {category['from']} import {category['name']}")

# Add the imported categories to __all__
__all__ = [category["name"] for category in categories if category["enable"]]



import argparse
import os
import requests
from zipfile import ZipFile
import subprocess
import shutil
import time
from aacommpy.nugetmanagement import download_nuget, nuget_version, update_nuget, dotnetfw, check_dotnet_versions
from settings import NET_FRAMEWORK_CHOICES


entry_points = {
    'console_scripts': [
        'nuget_download = nugetmanagement:download_nuget',
        'nuget_version = nugetmanagement:nuget_version',
        'update_nuget = nugetmanagement:update_nuget'
    ]
}
def install_template() -> None:
    msg = 'Download Python package for AAComm to this directory (y/n)? '
    user_response = input(msg)
    if user_response != 'y':
        return None

    directory = os.path.join(os.path.dirname(__file__), 'aacommpyDownloader-main')
    os.makedirs(directory, exist_ok=True)  # Create the directory if it doesn't exist

    url = 'https://dist.nuget.org/win-x86-commandline/latest/nuget.exe'
    r = requests.get(url)
    with open(os.path.join(directory, 'nuget.exe'), 'wb') as f:
        f.write(r.content)

    return None

def download_and_install(version: str = "") -> None:
    install_template()
    # Add code to wait for the nuget.exe file to be fully downloaded
    nuget_path = os.path.join(os.path.dirname(__file__), 'aacommpyDownloader-main', 'nuget.exe')
    while not os.path.exists(nuget_path):
        time.sleep(1)
    # nuget.exe is fully downloaded, proceed with download_nuget()
    if version != "":
        download_nuget(version)
    else:
        download_nuget()
    # Perform additional actions after download_nuget()
    available_versions = check_dotnet_versions()
    if available_versions is not None and len(available_versions) > 0:
        dotnetfw(available_versions[0])
    return None

def main() -> None:
    parser = argparse.ArgumentParser(description='Download aacommpy package.')
    parser.add_argument('command', choices=['install', 'downloadnuget', 'version', 'update', 'dotnetfw'], help='Choose a command to execute.')
    parser.add_argument('--version', help='Specify version to install/download.')
    parser.add_argument('--netfw', choices=NET_FRAMEWORK_CHOICES, default='net40', help='Choose the .NET framework version to use.')
    parser.add_argument('--check', action='store_true', help='Check compatibility versions of .NET framework.')
    args = parser.parse_args()

    if args.command == 'install':
        if args.version:
            download_and_install(args.version)
        else:
            download_and_install()
    elif args.command == 'version':
        nuget_version()
    elif args.command == 'update':
        update_nuget()
    elif args.command == 'dotnetfw':
        if args.check:
            check_dotnet_versions()
        else:
            dotnetfw(version=args.netfw)
    else:
        raise RuntimeError('Please supply a valid command for aacommpy - e.g. install.')

    return None

if __name__ == '__main__':
    main()
